import sys, unittest, socket
import inspect
from raygun4py import raygunmsgs

class TestRaygunMessageBuilder(unittest.TestCase):

    def setUp(self):
        self.builder = raygunmsgs.RaygunMessageBuilder().new()
        self.request = {
            "headers": {
                "referer": "localhost",
                "user-Agent": "Mozilla"
            },
            "hostName": "localhost",
            "url": "/resource",
            "httpMethod": "GET",
            "ipAddress": "127.0.0.1",
            "queryString": None,
            "form": None,
            "rawData": None
        }

    def test_machinename(self):
        self.builder.set_machine_name(socket.gethostname())
        self.assertIsNotNone(self.builder.raygunMessage.details['machineName'])

    def test_customdata(self):
        self.builder.set_customdata({1: "one"})
        self.assertIsInstance(self.builder.raygunMessage.details['userCustomData'], dict)

    def test_tags(self):
        self.builder.set_tags([1, 2, 3])
        self.assertIsInstance(self.builder.raygunMessage.details['tags'], list)

    def test_request_ip(self):
        self.builder.set_request_details(self.request)
        self.assertEqual(self.builder.raygunMessage.details['request']['iPAddress'], '127.0.0.1')

    def test_user_fname(self):
        self.builder.set_user({
            'firstName': 'Foo',

        })
        self.assertEqual(self.builder.raygunMessage.details['user']['firstName'], 'Foo')

    def test_user_fullname(self):
        self.builder.set_user({
            'fullName': 'Foo Bar',
        })
        self.assertEqual(self.builder.raygunMessage.details['user']['fullName'], 'Foo Bar')

    def test_user_email(self):
        self.builder.set_user({
            'email': 'foo@bar.com',
        })
        self.assertEqual(self.builder.raygunMessage.details['user']['email'], 'foo@bar.com')

    def test_user_identifier(self):
        self.builder.set_user({
            'identifier': 'foo@bar.com',
        })
        self.assertEqual(self.builder.raygunMessage.details['user']['identifier'], 'foo@bar.com')

    def test_user_anon(self):
        self.builder.set_user({
            'isAnonymous': False
        })
        self.assertEqual(self.builder.raygunMessage.details['user']['isAnonymous'], False)

class TestRaygunErrorMessage(unittest.TestCase):
    class GrandchildError(Exception):
        pass

    class ChildError(Exception):
        pass

    class ParentError(Exception):
        pass

    def setUp(self):
        try:
            self.parent()
        except Exception as e:
            self.theException = e

            exc_info = sys.exc_info()
            self.msg = raygunmsgs.RaygunErrorMessage(exc_info[0], exc_info[1], exc_info[2], { 'transmitLocalVariables': True })

    def parent(self):
            try:
                self.child()
            except TestRaygunErrorMessage.ChildError as exc:
                raise TestRaygunErrorMessage.ParentError("Parent message") from exc

    def child(self):
        try:
            raise TestRaygunErrorMessage.GrandchildError("Grandchild message")
        except Exception as ex:
            raise TestRaygunErrorMessage.ChildError("Child message")

    def test_exc_traceback_none_generates_empty_array(self):
        error_message = raygunmsgs.RaygunErrorMessage(Exception, None, None, {})
        self.assertEqual(error_message.stackTrace, [])

    def test_classname(self):
        self.assertEqual(self.msg.className, 'ParentError')

    def test_chained_exception_message_parent_has_nested_child(self):
        self.assertEqual(self.msg.innerError.className, 'ChildError')
        pass

    def test_chained_exception_message_child_has_nested_grandchild(self):
        self.assertEqual(self.msg.innerError.innerError.className, 'GrandchildError')
        pass

    def test_chained_exception_nested_child_message(self):
        self.assertEqual(self.msg.innerError.message, 'ChildError: Child message')
        pass

    def test_chained_exception_nested_grandchild_message(self):
        self.assertEqual(self.msg.innerError.innerError.message, 'GrandchildError: Grandchild message')
        pass

    def test_chained_exception_last_exception_caught_is_parent(self):
        self.assertIsInstance(self.theException.__context__, TestRaygunErrorMessage.ChildError)

    def test_chained_exception_cause_is_child(self):
        self.assertIsInstance(self.theException.__cause__, TestRaygunErrorMessage.ChildError)

    def test_chained_exception_childs_cause_is_grandchild(self):
        self.assertIsInstance(self.theException.__cause__.__context__, TestRaygunErrorMessage.GrandchildError)

    def test_methodname_none(self):
        original_getinnerframes = inspect.getinnerframes
        inspect.getinnerframes = getinnerframes_mock_methodname_none

        error_message = raygunmsgs.RaygunErrorMessage(int, 1, None, { "transmitLocalVariables": False })

        self.assertEqual(error_message.__dict__['stackTrace'][0]['methodName'], None)

        inspect.getinnerframes = original_getinnerframes

def getinnerframes_mock_methodname_none(exception):
    return [(
        'localVar',
        'fileName',
        'lineNumber',
        'className',
        None
    )]

def main():
    unittest.main()

if __name__ == '__main__':
    main()


import logging
import flask
from flask.signals import got_request_exception
from raygun4py import raygunprovider

log = logging.getLogger(__name__)


class Provider(object):
    def __init__(self, flaskApp, apiKey):
        self.flaskApp = flaskApp
        self.apiKey = apiKey

        got_request_exception.connect(self.send_exception, sender=flaskApp)

        flaskApp.extensions['raygun'] = self

    def attach(self):
        if not hasattr(self.flaskApp, 'extensions'):
            self.flaskApp.extensions = {}

        self.sender = raygunprovider.RaygunSender(self.apiKey)

    def send_exception(self, *args, **kwargs):
        if not self.sender:
            log.error("Raygun-Flask: Cannot send as provider not attached")

        env = self._get_flask_environment()
        self.sender.send_exception(extra_environment_data=env)

    def _get_flask_environment(self):
        return {
            'frameworkVersion': 'Flask ' + getattr(flask, '__version__', '')
        }



import sys
import unittest2 as unittest
import socket
import inspect

from raygun4py import raygunmsgs

class TestRaygunMessageBuilder(unittest.TestCase):

    def setUp(self):
        self.builder = raygunmsgs.RaygunMessageBuilder().new()
        self.request = {
            "headers": {
                "referer": "localhost",
                "user-Agent": "Mozilla"
            },
            "hostName": "localhost",
            "url": "/resource",
            "httpMethod": "GET",
            "ipAddress": "127.0.0.1",
            "queryString": None,
            "form": None,
            "rawData": None
        }

    def test_machinename(self):
        self.builder.set_machine_name(socket.gethostname())
        self.assertTrue(self.builder.raygunMessage.details['machineName'] != None)

    def test_customdata(self):
        self.builder.set_customdata({1: "one"})
        self.assertTrue(isinstance(self.builder.raygunMessage.details['userCustomData'], dict))

    def test_tags(self):
        self.builder.set_tags([1, 2, 3])
        self.assertTrue(isinstance(self.builder.raygunMessage.details['tags'], list))

    def test_request_ip(self):
        self.builder.set_request_details(self.request)
        self.assertEqual(self.builder.raygunMessage.details['request']['iPAddress'], '127.0.0.1')

    def test_user_fname(self):
        self.builder.set_user({
            'firstName': 'Foo',

        })
        self.assertEqual(self.builder.raygunMessage.details['user']['firstName'], 'Foo')

    def test_user_fullname(self):
        self.builder.set_user({
            'fullName': 'Foo Bar',
        })
        self.assertEqual(self.builder.raygunMessage.details['user']['fullName'], 'Foo Bar')

    def test_user_email(self):
        self.builder.set_user({
            'email': 'foo@bar.com',
        })
        self.assertEqual(self.builder.raygunMessage.details['user']['email'], 'foo@bar.com')

    def test_user_identifier(self):
        self.builder.set_user({
            'identifier': 'foo@bar.com',
        })
        self.assertEqual(self.builder.raygunMessage.details['user']['identifier'], 'foo@bar.com')

    def test_user_anon(self):
        self.builder.set_user({
            'isAnonymous': False
        })
        self.assertEqual(self.builder.raygunMessage.details['user']['isAnonymous'], False)

class TestRaygunErrorMessage(unittest.TestCase):
    class ParentError(Exception):
        pass

    def setUp(self):
        i_must_be_included = 'and i am'

        try:
            self.parent()
        except Exception as e:
            self.theException = e
            exc_info = sys.exc_info()
            self.msg = raygunmsgs.RaygunErrorMessage(exc_info[0], exc_info[1], exc_info[2], { 'transmitLocalVariables': True })

    def parent(self):
            raise TestRaygunErrorMessage.ParentError("Parent message")

    def test_exc_traceback_none_generates_empty_array(self):
        errorMessage = raygunmsgs.RaygunErrorMessage(int, 1, None, {})
        self.assertEqual(errorMessage.stackTrace, [])

    def test_classname(self):
        self.assertEqual(self.msg.className, 'ParentError')

    def test_local_variable(self):
        localVars = self.msg.__dict__['stackTrace'][0]['localVariables']

        self.assertTrue('i_must_be_included' in localVars)

    def test_methodname_none(self):
        originalGetinnerframes = inspect.getinnerframes
        inspect.getinnerframes = getinnerframes_mock_methodname_none

        errorMessage = raygunmsgs.RaygunErrorMessage(int, 1, None, { "transmitLocalVariables": False })

        self.assertEqual(errorMessage.__dict__['stackTrace'][0]['methodName'], None)

        inspect.getinnerframes = originalGetinnerframes

def getinnerframes_mock_methodname_none(exception):
    return [(
        'localVar',
        'fileName',
        'lineNumber',
        'className',
        None
    )]


def main():
    unittest.main()

if __name__ == '__main__':
    main()



import sys
import unittest2 as unittest
from raygun4py import raygunprovider
from raygun4py import raygunmsgs
from raygun4py import utilities


class TestRaygunSender(unittest.TestCase):

    def setUp(self):
        self.sender = raygunprovider.RaygunSender('invalidapikey')
        self.handler = raygunprovider.RaygunHandler('testkey', 'v1.0')

    def test_apikey(self):
        self.assertEqual(self.sender.api_key, 'invalidapikey')

    def test_handler_apikey(self):
        self.assertEqual(self.handler.sender.api_key, 'testkey')

    def test_handler_version(self):
        self.assertEqual(self.handler.version, 'v1.0')

    def test_sending_403_with_invalid_key(self):
        try:
            raise StandardError('test')
        except Exception as e:            
            info = sys.exc_info()
            http_result = self.sender.send_exception(info)
            self.assertEqual(http_result[0], 403)

    def test_ignore_exceptions(self):
        ex = ['Exception']
        self.sender.ignore_exceptions(ex)

        self.assertEqual(self.sender.ignored_exceptions, ex)

    def test_filter_keys_set(self):
        keys = ['credit_card']
        self.sender.filter_keys(keys)

        self.assertEqual(self.sender.filtered_keys, keys)

    def test_filter_keys_filters_error(self):
        keys = ['identifier']
        self.sender.filter_keys(keys)

        self.sender.set_user({ 'identifier': 'foo' })

        self.assertEqual(utilities.filter_keys(keys, self.sender.user)['identifier'], '<filtered>')

    def test_set_transmitLocalVariables(self):
        self.sender = raygunprovider.RaygunSender('foo', config={ 'transmit_local_variables': False })
        self.assertFalse(self.sender.transmit_local_variables)

    def test_set_transmitLocalVariables_camelcase(self):
        self.sender = raygunprovider.RaygunSender('foo', config={ 'transmitLocalVariables': False })
        self.assertFalse(self.sender.transmit_local_variables)

    def test_default_transmitLocalVariables(self):
        self.sender = raygunprovider.RaygunSender('foo')

        self.assertTrue(self.sender.transmit_local_variables)

    def test_set_transmit_global_variables(self):
        self.sender = raygunprovider.RaygunSender('foo', config={ 'transmit_global_variables': False })
        self.assertFalse(self.sender.transmit_global_variables)

    def test_set_transmit_global_variables_camelcase(self):
        self.sender = raygunprovider.RaygunSender('foo', config={ 'transmitGlobalVariables': False })
        self.assertFalse(self.sender.transmit_global_variables)

    def test_default_global_variables(self):
        self.sender = raygunprovider.RaygunSender('foo')
        self.assertTrue(self.sender.transmit_global_variables)


class TestGroupingKey(unittest.TestCase):

    def the_callback(self, raygun_message):
        return self.key

    def create_dummy_message(self):
        self.sender = raygunprovider.RaygunSender('apikey')

        msg = raygunmsgs.RaygunMessageBuilder().new()
        errorMessage = raygunmsgs.RaygunErrorMessage(Exception, None, None, {})
        msg.set_exception_details(errorMessage)
        return msg.build()

    def test_groupingkey_is_not_none_with_callback(self):
        msg = self.create_dummy_message()
        self.sender.on_grouping_key(self.the_callback)
        self.key = 'foo'
        self.sender._transform_message(msg)

        self.assertIsNotNone(msg.get_details()['groupingKey'])

    def test_groupingkey_is_set_with_callback(self):
        msg = self.create_dummy_message()
        self.sender.on_grouping_key(self.the_callback)
        self.key = 'foo'
        self.sender._transform_message(msg)

        self.assertEquals(msg.get_details()['groupingKey'], 'foo')

    def test_groupingkey_is_string_with_callback(self):
        msg = self.create_dummy_message()
        self.sender.on_grouping_key(self.the_callback)
        self.key = 'foo'
        self.sender._transform_message(msg)

        self.assertIsInstance(msg.get_details()['groupingKey'], str)

    def test_groupingkey_is_none_when_not_string_returned_from_callback(self):
        msg = self.create_dummy_message()
        self.sender.on_grouping_key(self.the_callback)
        self.key = object
        self.sender._transform_message(msg)

        self.assertIsNone(msg.get_details()['groupingKey'])

    def test_groupingkey_is_none_when_empty_string_returned_from_callback(self):
        msg = self.create_dummy_message()
        self.sender.on_grouping_key(self.the_callback)
        self.key = ''
        self.sender._transform_message(msg)

        self.assertIsNone(msg.get_details()['groupingKey'])

    def test_groupingkey_is_set_when_ok_length_string_returned_from_callback(self):
        msg = self.create_dummy_message()
        self.sender.on_grouping_key(self.the_callback)
        self.key = 'a'

        for i in range(0, 99):
            self.key += 'a'

        self.sender._transform_message(msg)
        self.assertEqual(msg.get_details()['groupingKey'], self.key)

    def test_groupingkey_is_none_when_too_long_string_returned_from_callback(self):
        msg = self.create_dummy_message()
        self.sender.on_grouping_key(self.the_callback)
        self.key = 'a'

        for i in range(0, 100):
            self.key += 'a'

        self.sender._transform_message(msg)
        self.assertIsNone(msg.get_details()['groupingKey'])


def main():
    unittest.main()

if __name__ == '__main__':
    main()



import sys, os, urllib2
import traceback
from raygun4py import raygunprovider

def handle_exception(exc_type, exc_value, exc_traceback):
    cl = raygunprovider.RaygunSender("paste_your_api_key_here")
    cl.set_version("1.0.0.0")
    cl.set_user({
        'identifier': 'example@email_or_user_id.com',
        'firstName': 'John',
        'fullName': 'John Smith',
        'email': 'example@email_or_user_id.com'
    })

    # If we're handling a request in a web server environment, we can send the HTTP request data
    headers = {
        "referer": "localhost",
        "user-Agent": "Mozilla"
    }

    request = {
        "headers": headers,
        "hostName": "localhost",
        "url": "/resource",
        "httpMethod": "GET",
        "ipAddress": "127.0.0.1",
        "queryString": None,
        "form": None,
        "rawData": None,
    }

    print cl.send_exception(exc_info=(exc_type, exc_value, exc_traceback), "myclass", ["tag1", "tag2"], {"key1": 1111, "key2": 2222}, request)

def very_buggy_request():
    methodtwo()

def methodtwo():
    raise StandardError("Test exception sent from raygun4py")

sys.excepthook = handle_exception

very_buggy_request()



import mock
import django
from django.conf import settings
from django.test.client import RequestFactory
from django.test import SimpleTestCase
from raygun4py.middleware.django import Provider

settings.configure(DEBUG=True, RAYGUN4PY_API_KEY='foo', ALLOWED_HOSTS=['testserver'])
django.setup()


class DjangoProviderTests(SimpleTestCase):

    def setUp(self):
        request_factory = RequestFactory()
        self.get_request = request_factory.get('/foo')

    def test_map_request(self):
        provider = Provider()
        request_payload = provider._mapRequest(self.get_request)
        self.assertEqual(request_payload['url'], '/foo')
        self.assertEqual(request_payload['httpMethod'], 'GET')

    def test_get_django_environment(self):
        provider = Provider()
        environment_payload = provider._get_django_environment()
        self.assertEqual(environment_payload['frameworkVersion'], django.get_version())

    def test_process_exception_called(self):
        provider = Provider()
        provider.sender = mock.MagicMock(name='send_exception')

        try:
            raise Exception
        except Exception as e:
            provider.process_exception(self.get_request, e)

        self.assertEqual(len(provider.sender.mock_calls), 1)



import sys, logging, os
from raygun4py import raygunprovider

logger = logging.getLogger("mylogger")

rgHandler = raygunprovider.RaygunHandler("paste_your_api_key_here")

logger.addHandler(rgHandler)

def log_exception(exc_type, exc_value, exc_traceback):
    logger.error("A python error occurred", exc_info = (exc_type, exc_value, exc_traceback))
    print "Logging: %s" % exc_value

sys.excepthook = log_exception

def buggyMethod():
    raise StandardError("Test exception sent via built-in handler")

buggyMethod()


import logging

from raygun4py import raygunprovider


log = logging.getLogger(__name__)


class Provider(object):

    def __init__(self, app, apiKey):
        self.app = app
        self.sender = raygunprovider.RaygunSender(apiKey)

    def __call__(self, environ, start_response):
        if not self.sender:
            log.error("Raygun-WSGI: Cannot send as provider not attached")

        iterable = None

        try:
            iterable = self.app(environ, start_response)
            for event in iterable:
                yield event

        except Exception as e:
            request = self.build_request(environ)
            self.sender.send_exception(exception=e, request=request)
            raise

        finally:
            if hasattr(iterable, 'close'):
                try:
                    iterable.close()
                except Exception as e:
                    request = self.build_request(environ)
                    self.sender.send_exception(exception=e, request=request)
                    raise

    def build_request(self, environ):
        request = {}

        http_host = environ.get(['HTTP_HOST'], None)
        if http_host is not None:
            http_host = http_host.replace(' ', '')

        try:
            request = {
                'httpMethod': environ.get('REQUEST_METHOD', None),
                'url': environ.get('PATH_INFO', None),
                'ipAddress': environ.get('REMOTE_ADDR', None),
                'hostName': http_host,
                'queryString': environ.get('QUERY_STRING', None),
                'headers': {},
                'form': {},
                'rawData': {}
            }
        except Exception:
            pass

        for key, value in environ.items():
            if key.startswith('HTTP_'):
               request['headers'][key] = value

        return request



import django
from django.conf import settings
from raygun4py import raygunprovider
from django.utils.functional import cached_property

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object

class Provider(MiddlewareMixin):

    def __init__(self, get_response=None):
        self.get_response = get_response
        config = getattr(settings, 'RAYGUN4PY_CONFIG', {})
        apiKey = getattr(settings, 'RAYGUN4PY_API_KEY', config.get('api_key', None))

        self.sender = raygunprovider.RaygunSender(apiKey, config=config)

    def process_exception(self, request, exception):
        raygun_request = self._mapRequest(request)
        env = self._get_django_environment()

        self.sender.send_exception(exception=exception, request=raygun_request, extra_environment_data=env)

    def _mapRequest(self, request):
        headers = request.META.items()
        _headers = dict()
        for k, v in headers:
            if not k.startswith('wsgi'):
                _headers[k] = v

        return {
            'hostName': request.get_host(),
            'url': request.path,
            'httpMethod': request.method,
            'ipAddress': request.META.get('REMOTE_ADDR', '?'),
            'queryString': dict((key, request.GET[key]) for key in request.GET),
            'form': dict((key, request.POST[key]) for key in request.POST),
            'headers': _headers,
            'rawData': request.body if hasattr(request, 'body') else getattr(request, 'raw_post_data', {})
        }

    def _get_django_environment(self):
        return {
            'frameworkVersion': django.get_version()
        }



import re
from raygun4py import raygunmsgs


def ignore_exceptions(ignored_exceptions, message):
    if message.get_error().get_classname() in ignored_exceptions:
        return None

    return message


def filter_keys(filtered_keys, object):
    iteration_target = object

    if isinstance(object, raygunmsgs.RaygunMessage):
        iteration_target = object.__dict__

    for key in iter(iteration_target.keys()):
        if key in filtered_keys:
            iteration_target[key] = '<filtered>'

        elif isinstance(iteration_target[key], dict):
            iteration_target[key] = filter_keys(filtered_keys, iteration_target[key])

    return iteration_target


def execute_grouping_key(grouping_key_callback, message):
    if grouping_key_callback is not None:
        grouping_key = grouping_key_callback(message)

        if grouping_key is not None and isinstance(grouping_key, str) and 0 < len(grouping_key) <= 100:
            return grouping_key

    return None


def camelcase_to_snakecase(key):
    "Turns camelCaseStrings into snake_case_strings."
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', key)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()


def snakecase_dict(d):
    return dict([
        (camelcase_to_snakecase(k), v) for k, v in d.items()
    ])



import sys
import copy
import socket
import logging
import jsonpickle
import requests
import requests.packages.urllib3
from raygun4py import raygunmsgs
from raygun4py import utilities

requests.packages.urllib3.disable_warnings()

DEFAULT_CONFIG = {
    'before_send_callback': None,
    'grouping_key_callback': None,
    'filtered_keys': [],
    'ignored_exceptions': [],
    'proxy': None,
    'transmit_global_variables': True,
    'transmit_local_variables': True,
    'userversion': "Not defined",
    'user': None,
    'http_timeout': 10.0
}


class RaygunSender:

    log = logging.getLogger(__name__)

    api_key = None
    endpointprotocol = 'https://'
    endpointhost = 'api.raygun.io'
    endpointpath = '/entries'

    def __init__(self, api_key, config=None):
        if (api_key):
            self.api_key = api_key
        else:
            self.log.warning("RaygunProvider error: ApiKey not set, errors will not be transmitted")

        try:
            import ssl
        except ImportError:
            self.log.warning("RaygunProvider error: No SSL support available, cannot send. Please"
                        "compile the socket module with SSL support.")

        # Set up the default values
        default_config = utilities.snakecase_dict(copy.deepcopy(DEFAULT_CONFIG))
        default_config.update(utilities.snakecase_dict(config or {}))
        for k, v in default_config.items():
            setattr(self, k, v)

    def set_version(self, version):
        if isinstance(version, basestring):
            self.userversion = version

    def set_user(self, user):
        self.user = user

    def ignore_exceptions(self, exceptions):
        if isinstance(exceptions, list):
            self.ignored_exceptions = exceptions

    def filter_keys(self, keys):
        if isinstance(keys, list):
            self.filtered_keys = keys

    def set_proxy(self, host, port):
        self.proxy = {
            'host': host,
            'port': port
        }

    def on_before_send(self, callback):
        if callable(callback):
            self.before_send_callback = callback

    def on_grouping_key(self, callback):
        if callable(callback):
            self.grouping_key_callback = callback

    def send_exception(self, exception=None, exc_info=None, **kwargs):
        options = {
            'transmitLocalVariables': self.transmit_local_variables,
            'transmitGlobalVariables': self.transmit_global_variables
        }

        if exc_info is None:
            exc_info = sys.exc_info()

        exc_type, exc_value, exc_traceback = exc_info

        if exception is not None:
            errorMessage = raygunmsgs.RaygunErrorMessage(type(exception), exception, exc_traceback, options)
        else:
            errorMessage = raygunmsgs.RaygunErrorMessage(exc_type, exc_value, exc_traceback, options)

        # Prevent circular reference for GC - https://docs.python.org/2/library/sys.html#sys.exc_info
        try:
            del exc_type, exc_value, exc_traceback
        except Exception as e:
            raise

        tags, custom_data, http_request, extra_environment_data = self._parse_args(kwargs)
        message = self._create_message(errorMessage, tags, custom_data, http_request, extra_environment_data)
        message = self._transform_message(message)

        if message is not None:
            return self._post(message)

    def _parse_args(self, kwargs):
        tags = kwargs['tags'] if 'tags' in kwargs else None
        custom_data = kwargs['userCustomData'] if 'userCustomData' in kwargs else None
        extra_environment_data = kwargs['extra_environment_data'] if 'extra_environment_data' in kwargs else None

        http_request = None
        if 'httpRequest' in kwargs:
            http_request = kwargs['httpRequest']
        elif 'request' in kwargs:
            http_request = kwargs['request']

        return tags, custom_data, http_request, extra_environment_data

    def _create_message(self, raygunExceptionMessage, tags, user_custom_data, http_request, extra_environment_data):
        return raygunmsgs.RaygunMessageBuilder().new() \
            .set_machine_name(socket.gethostname()) \
            .set_version(self.userversion) \
            .set_client_details() \
            .set_exception_details(raygunExceptionMessage) \
            .set_environment_details(extra_environment_data) \
            .set_tags(tags) \
            .set_customdata(user_custom_data) \
            .set_request_details(http_request) \
            .set_user(self.user) \
            .build()

    def _transform_message(self, message):
        message = utilities.ignore_exceptions(self.ignored_exceptions, message)

        if message is not None:
            message = utilities.filter_keys(self.filtered_keys, message)
            message['details']['groupingKey'] = utilities.execute_grouping_key(self.grouping_key_callback, message)

        if self.before_send_callback is not None:
            mutated_payload = self.before_send_callback(message['details'])

            if mutated_payload is not None:
                message['details'] = mutated_payload
            else:
                return None

        return message

    def _post(self, raygunMessage):
        json = jsonpickle.encode(raygunMessage, unpicklable=False)

        try:
            headers = {
                "X-ApiKey": self.api_key,
                "Content-Type": "application/json",
                "User-Agent": "raygun4py"
            }

            response = requests.post(self.endpointprotocol + self.endpointhost + self.endpointpath,
                                     headers=headers, data=json, timeout=self.http_timeout)
        except Exception as e:
            self.log.error(e)
            return 400, "Exception: Could not send"
        return response.status_code, response.text


class RaygunHandler(logging.Handler):
    def __init__(self, api_key, version=None):
        logging.Handler.__init__(self)
        self.sender = RaygunSender(api_key)
        self.version = version

    def emit(self, record):
        userCustomData = {
            "Logger Message": record.msg
        }
        self.sender.send_exception(userCustomData=userCustomData)



# coding=utf-8

import unittest2 as unittest
import sys, logging, socket, os
from raygun4py import raygunprovider

class TestRaygun4PyFunctional(unittest.TestCase):

    def setUp(self):
        self.apiKey = "kImNMh/h98JZ233PUKv87g=="

    def test_send_with_user(self):
        client = raygunprovider.RaygunSender(self.apiKey)
        client.set_user({
            'firstName': 'foo',
            'fullName': 'foo bar',
            'email': 'foo@bar.com',
            'isAnonymous': False,
            'identifier': 'foo@bar.com'
          })

        try:
            raise Exception("Raygun4py manual sending test - user")
        except:
            httpResult = client.send_exception(exc_info=sys.exc_info())

            self.assertEqual(httpResult[0], 202)

    def test_send_with_version(self):
        client = raygunprovider.RaygunSender(self.apiKey)
        client.set_version('v1.0.0')

        try:
            raise Exception("Raygun4py manual sending test - version")
        except:
            httpResult = client.send_exception(exc_info=sys.exc_info())

            self.assertEqual(httpResult[0], 202)

    def test_send_with_tags(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        try:
            raise Exception("Raygun4py manual sending test - tags")
        except:
            httpResult = client.send_exception(exc_info=sys.exc_info(), tags=["I am a tag"])

            self.assertEqual(httpResult[0], 202)

    def log_send(self, logger):
        try:
            raise StandardError("Raygun4py Logging Test")
        except:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            logger.error("Logging with sending", exc_info = (exc_type, exc_value, exc_traceback))
            return 0

        return 1

    def log_nosend(self, logger):
        try:
            raise StandardError("Raygun4py Logging Test")
        except:
            logger.error("Logging without sending")
            return 0

        return 1

    def test_log_with_sending(self):
        logger = logging.getLogger("mylogger")
        rgHandler = raygunprovider.RaygunHandler(self.apiKey)
        logger.addHandler(rgHandler)

        self.assertEquals(0, self.log_send(logger))


    def test_log_without_sending(self):
        logger = logging.getLogger("mylogger")
        rgHandler = raygunprovider.RaygunHandler(self.apiKey)
        logger.addHandler(rgHandler)

        self.assertEquals(0, self.log_nosend(logger))

    def test_send_exception_no_args(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        try:
            raise Exception("Raygun4py functional test - Py2 send_exception")
        except:
            httpResult = client.send_exception()

            self.assertEqual(httpResult[0], 202)

    def test_send_exception_with_exc_info(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        try:
            raise StandardError("Raygun4py manual sending test")
        except:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            httpResult = client.send_exception(exc_info = sys.exc_info())


    def test_send_exception(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        try:
            raise Exception("Raygun4py functional test - Py2 send_exception")
        except Exception as e:
            httpResult = client.send_exception(e)

            self.assertEqual(httpResult[0], 202)

    def test_send_exception_with_exc_info(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        try:
            raise Exception("Raygun4py functional test - Py2 send_exception with exc_info")
        except Exception as e:
            httpResult = client.send_exception(e, exc_info = sys.exc_info())

            self.assertEqual(httpResult[0], 202)

    def test_send_exception_subclass(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        try:
            raise CustomException("Raygun4py functional test - Py2 send_exception with custom exception")
        except CustomException as e:
            httpResult = client.send_exception(e)

            self.assertEqual(httpResult[0], 202)

    def test_ignore_exception(self):
        client = raygunprovider.RaygunSender(self.apiKey)
        client.ignore_exceptions(['CustomException'])

        try:
            raise CustomException("This test should not send an exception")
        except CustomException as e:
            httpResult = client.send_exception(e)

            self.assertEqual(httpResult, None)

    def test_filter_keys_202(self):
        client = raygunprovider.RaygunSender(self.apiKey)
        client.filter_keys(['environment'])

        try:
            raise Exception("Raygun4py functional test - Py2 filter_keys")
        except Exception as e:
            httpResult = client.send_exception(e, exc_info = sys.exc_info())

        self.assertEqual(httpResult[0], 202)

    @unittest.skip('Requires a proxy, skipping for Travis')
    def test_proxy(self):
        client = raygunprovider.RaygunSender(self.apiKey)
        client.set_proxy('127.0.0.1', 3128)

        try:
            raise Exception("Raygun4py functional test - Py2 set_proxy")
        except Exception as e:
            httpResult = client.send_exception(e, exc_info = sys.exc_info())

        self.assertEqual(httpResult[0], 202)


    def test_before_send_callback(self):
        client = raygunprovider.RaygunSender(self.apiKey)
        client.on_before_send(before_send_mutate_payload)

        try:
            raise Exception("Raygun4py functional test - on_before_send")
        except Exception as e:
            httpResult = client.send_exception(e, exc_info = sys.exc_info())

        self.assertEqual(httpResult[0], 202)

    def test_before_send_callback_sets_none_cancels_send(self):
        client = raygunprovider.RaygunSender(self.apiKey)
        client.on_before_send(before_send_cancel_send)

        try:
            raise Exception("Raygun4py functional test - on_before_send")
        except Exception as e:
            result = client.send_exception(e, exc_info = sys.exc_info())

        self.assertIsNone(result)

    def test_request(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        try:
            raise Exception("Raygun4py functional test - on_before_send")
        except Exception as e:
            result = client.send_exception(request={})

            self.assertEqual(result[0], 202)

    def test_http_request(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        try:
            raise Exception("Raygun4py functional test - on_before_send")
        except Exception as e:
            result = client.send_exception(httpRequest={})

            self.assertEqual(result[0], 202)

    def test_localvariables(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        try:
            foo = 'bar'
            raise Exception("Raygun4py functional test - local variables")
        except Exception as e:
            result = client.send_exception(httpRequest={})

            self.assertEqual(result[0], 202)

    def test_localvariables_multilevels(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        try:
            scope = 'parent'
            child()
        except Exception as e:
            result = client.send_exception(httpRequest={})

            self.assertEqual(result[0], 202)

    def test_utf8_message(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        try:
            raise Exception("ΔΔΔΔ")
        except Exception as e:
            result = client.send_exception(httpRequest={})

            self.assertEqual(result[0], 202)

    def test_utf8_localvariable(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        the_variable = 'ᵫ'

        try:
            raise Exception("Raygun4py2: utf8 local variable")
        except Exception as e:
            result = client.send_exception(httpRequest={})

            self.assertEqual(result[0], 202)

    def test_bytestring_localvariable(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        byte_string = b'\x8d\x80\x92uK!M\xed'

        try:
            raise Exception("Raygun4py2: bytestring local variable")
        except Exception as e:
            result = client.send_exception(httpRequest={})

            self.assertEqual(result[0], 202)

    def test_localvariables_unicode(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        try:
            sigma = u'\u2211'
            raise Exception("Raygun4py functional test - local variable - unicode")
        except Exception as e:
            result = client.send_exception(httpRequest={})

            self.assertEqual(result[0], 202)

    def test_localvariables_cause_str_exception(self):
        client = raygunprovider.RaygunSender(self.apiKey)

        class StrFailingClass(object):
            def __str__(self):
                raise Exception("I failed to stringify myself")

        instance = StrFailingClass()

        try:
            raise Exception("Raygun4py functional test - local variable - cause an str exception")
        except Exception as e:
            result = client.send_exception(httpRequest={})

            self.assertEqual(result[0], 202)

class CustomException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

def before_send_mutate_payload(message):
    message['newKey'] = 'newValue'
    return message

def before_send_cancel_send(message):
    return None

def child():
    throwerScope = 'child'
    raise Exception("Raygun4py functional test - local variables multi levels")


import sys
from optparse import OptionParser
from raygun4py import raygunprovider


def main():
    usage = '\n  raygun4py test <apikey>'
    parser = OptionParser(usage=usage)

    options, args = parser.parse_args()

    if 'install' in args:
        if len(args) > 1:
            print("Installed API key! Now run 'raygun4py test' to check it's working")
        else:
            print('Please provide a Raygun API key!')
    elif 'test' in args:
        if len(args) > 1 and isinstance(args[1], str):
            send_test_exception(args[1])
        else:
            print('Please provide your API key')
    else:
        parser.print_help()


def send_test_exception(apikey):
    client = raygunprovider.RaygunSender(apikey)

    try:
        raise Exception("Test exception from Raygun4py3!")
    except:
        response = client.send_exception()

        if response[0] is 202:
            print("Success! Now check your Raygun dashboard at https://app.raygun.io")
        else:
            print("Something went wrong - please check your API key or contact us to get help. The response was:")
            print(response)



import sys
import os
import inspect
import jsonpickle

try:
    import multiprocessing
    USE_MULTIPROCESSING = True
except ImportError:
    USE_MULTIPROCESSING = False

import platform
from datetime import datetime


class RaygunMessageBuilder(object):

    def __init__(self):
        self.raygunMessage = RaygunMessage()

    def new(self):
        return RaygunMessageBuilder()

    def build(self):
        return self.raygunMessage

    def set_machine_name(self, name):
        self.raygunMessage.details['machineName'] = name
        return self

    def set_environment_details(self, extra_environment_data):
        self.raygunMessage.details['environment'] = {
            "environmentVariables": dict(os.environ),
            "runtimeLocation": sys.executable,
            "runtimeVersion": 'Python ' + sys.version
        }

        # Wrap these so we gracefully fail if we cannot access the system details for any reason
        try:
            self.raygunMessage.details['environment']["processorCount"] = (
                multiprocessing.cpu_count() if USE_MULTIPROCESSING else "n/a"
            )
        except Exception: # pragma: no cover
            pass

        try:
            self.raygunMessage.details['environment']["architecture"] = platform.architecture()[0]
        except Exception: # pragma: no cover
            pass

        try:
            self.raygunMessage.details['environment']["cpu"] = platform.processor()
        except Exception: # pragma: no cover
            pass

        try:
            self.raygunMessage.details['environment']["oSVersion"] = "%s %s" % \
                (platform.system(), platform.release())
        except Exception: # pragma: no cover
            pass

        if extra_environment_data is not None:
            merged = extra_environment_data.copy()
            merged.update(self.raygunMessage.details['environment'])
            self.raygunMessage.details['environment'] = merged

        return self

    def set_exception_details(self, raygunExceptionMessage):
        self.raygunMessage.details['error'] = raygunExceptionMessage
        return self

    def set_client_details(self):
        self.raygunMessage.details['client'] = {
            "name": "raygun4py",
            "version": "3.1.3",
            "clientUrl": "https://github.com/MindscapeHQ/raygun4py"
        }
        return self

    def set_customdata(self, user_custom_data):
        if type(user_custom_data) is dict:
            self.raygunMessage.details['userCustomData'] = user_custom_data
        return self

    def set_tags(self, tags):
        if type(tags) is list or tuple:
            self.raygunMessage.details['tags'] = tags
        return self

    def set_request_details(self, request):
        if request:
            self.raygunMessage.details['request'] = {
              "hostName": request['hostName'],
              "url": request['url'],
              "httpMethod": request['httpMethod'],
              "queryString": request['queryString'],
              "form": request['form'],
              "headers": request['headers'],
              "rawData": request['rawData']
            }

            if 'ipAddress' in request:
                self.raygunMessage.details['request']['iPAddress'] = request['ipAddress']
            elif 'iPAddress' in request:
                self.raygunMessage.details['request']['iPAddress'] = request['iPAddress']

        return self

    def set_version(self, version):
        self.raygunMessage.details['version'] = version
        return self

    def set_user(self, user):
        if user is not None:
            self.raygunMessage.details['user'] = user
        return self


class RaygunMessage(object):

    def __init__(self):
        self.occurredOn = datetime.utcnow()
        self.details = {}

    def get_error(self):
        return self.details['error']

    def get_details(self):
        return self.details


class RaygunErrorMessage(object):

    def __init__(self, exc_type, exc_value, exc_traceback, options):
        self.className = exc_type.__name__
        self.message = "%s: %s" % (exc_type.__name__, exc_value)
        self.stackTrace = []

        try:
            frames = inspect.getinnerframes(exc_traceback)

            if frames:
                for frame in frames:
                    localVariables = None
                    if 'transmitLocalVariables' in options and options['transmitLocalVariables'] is True:
                        localVariables = self._get_locals(frame[0])

                    self.stackTrace.append({
                        'lineNumber': frame[2],
                        'className': frame[3],
                        'fileName': frame[1],
                        'methodName': frame[4][0] if frame[4] is not None else None,
                        'localVariables': localVariables
                    })
                if 'transmitGlobalVariables' in options and options['transmitGlobalVariables'] is True and len(frames) > 0:
                    self.globalVariables = frames[-1][0].f_globals

        finally:
            del frames

        self.data = ""

        if isinstance(exc_value, Exception):
            nestedException = None

            if exc_value.__cause__:
                nestedException = exc_value.__cause__
            elif exc_value.__context__:
                nestedException = exc_value.__context__

            if nestedException is not None:
                self.innerError = RaygunErrorMessage(type(nestedException), nestedException, nestedException.__traceback__, options)

        try:
            jsonpickle.encode(self, unpicklable=False)
        except Exception as e:
            if self.globalVariables:
                self.globalVariables = None

                try:
                    jsonpickle.encode(self, unpicklable=False)
                except Exception as e:
                    for frame in self.stackTrace:
                        if 'localVariables' in frame:
                            frame['localVariables'] = None

    def get_classname(self):
        return self.className

    def _get_locals(self, frame):
        result = {}
        localVars = getattr(frame, 'f_locals', {})

        if '__traceback_hide__' not in localVars:
            for key in localVars:
                try:
                    # Note that str() *can* fail; thus protect against it as much as we can.
                    result[key] = str(localVars[key])
                except Exception as e:
                    try:
                        r = repr(localVars[key])
                    except Exception as re:
                        r = "Couldn't convert to repr due to {0}".format(re)
                    result[key] = "!!! Couldn't convert {0!r} (repr: {1}) due to {2!r} !!!".format(key, r, e)
            return result



import sys
from setuptools import setup

packages = ['raygun4py', 'raygun4py.middleware']

if sys.version_info[0] == 2:
    base_dir = 'python2'
elif sys.version_info[0] == 3:
    base_dir = 'python3'

setup(
    name='raygun4py',
    version='3.1.6',
    packages=packages,
    package_dir= {
        "raygun4py": base_dir + "/raygun4py"
    },
    license='LICENSE',
    url='https://raygun.io',
    author='Raygun',
    author_email='hello@raygun.io',
    description='Official Raygun provider for Python 2.6/2.7 and Python 3+',
    long_description=open('README.rst').read(),
    install_requires=[
        'jsonpickle >= 0.9.2',
        'blinker >= 1.3.0',
        'requests >= 2.9.1'
    ],
    entry_points={
        'console_scripts': [
            'raygun4py = raygun4py.cli:main'
        ]
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Communications',
    ],
)



import sys, logging, os
from raygun4py import raygunprovider

# Hook the Raygun logging handler up:

logger = logging.getLogger("mylogger")
rgHandler = raygunprovider.RaygunHandler("paste_your_api_key_here")
logger.addHandler(rgHandler)

def log_exception(exc_type, exc_value, exc_traceback):
    logger.error("A python error occurred", exc_info=(exc_type, exc_value, exc_traceback))
    print("Logging: %s" % exc_value)

sys.excepthook = log_exception

## Example exception:

def buggyMethod():
    raise Exception("Test exception sent via built-in handler")

buggyMethod()